part of masamune_agora;

/// Manage Agora RTC.
///
/// You can get [uid] and [name] by executing [initialize()].
class AgoraRTCCore {
  AgoraRTCCore._();

  /// Manage Agora RTC.
  ///
  /// You can get [uid] and [name] by executing [initialize()].
  static final AgoraRTCCore instance = AgoraRTCCore._();
  static bool get isInitialized => _isInitialized;
  static bool _isInitialized = false;

  late final RtcEngine _engine;

  /// App ID.
  late final String appId;

  /// Customer ID.
  late final String? customerId;
  late final String? _customerSecret;
  late final AgoraStorageBucketConfig? _storageBucketConfig;

  /// User UID.
  late final int uid;

  /// The user's name.
  late final String name;

  /// Manage Agora RTC.
  ///
  /// You can get [uid] and [name] by executing [initialize()].
  static Future<AgoraRTCCore> initialize({
    required String appId,
    required String userName,
    String? customerId,
    String? customerSecret,
    Duration timeout = const Duration(seconds: 60),
    AgoraStorageBucketConfig? storageBucketConfig,
  }) async {
    try {
      final agora = instance;
      if (isInitialized) {
        return agora;
      }
      agora.appId = appId;
      agora.customerId = customerId;
      agora._customerSecret = customerSecret;
      agora._storageBucketConfig = storageBucketConfig;
      agora._engine = await RtcEngine.create(appId).timeout(timeout);
      agora._engine.setEventHandler(
          RtcEngineEventHandler(localUserRegistered: (uid, name) {
        agora.name = name;
        agora.uid = uid;
      }));
      await agora._engine
          .registerLocalUserAccount(appId, userName)
          .timeout(timeout);
      _isInitialized = true;
      return agora;
    } catch (e) {
      rethrow;
    }
  }
}
